﻿using FObjBase;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace FLY.Thick.Blowing.IService
{
    /// <summary>
    /// 旋转架及膜位置检测模拟
    /// </summary>
    public interface IBlowingDetect:INotifyPropertyChanged
    {
        #region 参数

        /// <summary>
        /// 离开限位 到 撞下一个限位 的 旋转架转动总角度 单位°
        /// </summary>
        double RAngle { get; set; }

        /// <summary>
        /// 人字架 旋转1周 设置 时间。 
        /// 刚开机时，RenZiJiaPeriod = DefaultRPeriod；
        /// 异常时，RenZiJiaPeriod = DefaultRPeriod；
        /// </summary>
        TimeSpan DefaultRPeriod { get; set; }

        /// <summary>
        /// 加速减速时间，  默认1s  加速减速时间 * 2 +正反转冷却时间 = 撞限位 到 离开限位 的时间
        /// </summary>
        TimeSpan AccDecTime { get; set; }

        /// <summary>
        /// 信号0 撞2次。 这个能自动设置。 两个撞同一信号 间隔30秒内，算一个换向信号。 
        /// 当为2次撞时：  StartTime = 第1次撞的StartTime， EndTime = 第2次撞的EndTime
        /// </summary>
        bool IsSign0Double { get; set; }

        /// <summary>
        /// 信号1 撞2次。 这个能自动设置。 两个撞同一信号 间隔30秒内，算一个换向信号。 
        /// 当为2次撞时：  StartTime = 第1次撞的StartTime， EndTime = 第2次撞的EndTime
        /// </summary>
        bool IsSign1Double { get; set; }

        /// <summary>
        /// 信号模式， 短信号模式（只撞一下），长信号模式（只要是转着，信号就长亮，直到换向）
        /// </summary>
        BlowingSignType SignType { get; set; }

        /// <summary>
        /// 撞限位 到 离开限位 的时间， 需要初始值，以后测量出来的
        /// </summary>
        TimeSpan LimitSignTime { get; set; }

        /// <summary>
        /// 人字架到测厚仪膜长 单位m
        /// </summary>
        double FilmLength { get; set; }

        /// <summary>
        /// 辊周长，单位mm
        /// </summary>
        double RollPerimeter { get; set; }

        /// <summary>
        /// 转向信号100ms 滤波
        /// </summary>
        bool IsSignFilter { get; set; }

        /// <summary>
        /// 缺少信号模式
        /// 正反信号线接得不好，有时候会丢失了。 
        /// 当前的旋转时间是上一次的1.1倍时，在以前的位置增加信号！！！！
        /// 当有连续两次相同的信号，且信号差刚好是旋转时间的1.9~2.1倍时，证明真的是缺少信号
        /// </summary>
        bool IsLackSignMode { get; set; }

        #endregion

        #region 状态
        /// <summary>
        /// 当前旋转架旋转角度
        /// </summary>
        double Angle { get; }

        /// <summary>
        /// 当前方向
        /// </summary>
        Misc.DIRECTION Direction { get; }

        /// <summary>
        /// 旋转次数
        /// </summary>
        int RotationCnt { get; }

        /// <summary>
        /// 最新的限位信号序号
        /// </summary>
        int LastLimitNo { get; }

        /// <summary>
        /// 最后一次撞限位，或离开限位信号 到当前已经消耗的时间
        /// </summary>
        TimeSpan PastTime { get; }

        /// <summary>
        /// 当前线速度,单位 m/min
        /// </summary>
        double FilmVelocity { get; }

        /// <summary>
        /// 离开限位 到 撞下一个限位  的 旋转架转动时间， 需要初始值，以后测量出来的
        /// </summary>
        TimeSpan RenZiJiaPeriod { get; }

        /// <summary>
        /// 正反转冷却时间， 加速减速时间 * 2 +正反转冷却时间 = 撞限位 到 离开限位 的时间 
        /// </summary>
        TimeSpan SwapCoolTime { get; }

        #endregion

        #region 信号列表
        /// <summary>
        /// 信号列表第一个记录序号
        /// </summary>
        int FirstSignBM { get; }
        /// <summary>
        /// 信号列表最后一个记录序号
        /// </summary>
        int LastSignBM { get; }

        #endregion

        #region 功能
        void Apply();
        /// <summary>
        /// 获取 信号列表
        /// </summary>
        /// <param name="AsyncDelegate">retdata = GetBufListReponse</param>
        /// <param name="AsyncState"></param>
        void GetSignList(AsyncCBHandler AsyncDelegate, object AsyncState);

        #endregion
    }


    /// <summary>
    /// 正反转信号类型
    /// </summary>
    public enum BlowingSignType
    {
        /// <summary>
        /// 短信号模式， 撞输入口 为0， 离开输入口为1
        /// </summary>
        Short,
        /// <summary>
        /// 长信号模式，撞输入口 为0，直到切换为下一个信号，才恢复为1
        /// </summary>
        Long
    }

    /// <summary>
    /// 信号
    /// </summary>
    public class SignData : IPack, INotifyPropertyChanged
    {
        private int no;
        /// <summary>
        /// 信号序号
        /// </summary>
        public int No
        {
            get { return no; }
            set
            {
                no = value;
                NotifyPropertyChanged("No");
            }

        }

        private DateTime dt;
        /// <summary>
        /// 时间
        /// </summary>
        public DateTime Time
        {
            get { return dt; }
            set
            {
                if (dt != value)
                {
                    dt = value;
                    NotifyPropertyChanged("Time");
                }
            }
        }

        private bool on;
        /// <summary>
        /// on? off?
        /// </summary>
        public bool On
        {
            get { return on; }
            set
            {
                if (on != value)
                {
                    on = value;
                    NotifyPropertyChanged("No");
                }
            }
        }

        private TimeSpan interval;
        /// <summary>
        /// 与上一次的时间间隔
        /// </summary>
        public TimeSpan Interval
        {
            get { return interval; }
            set
            {
                if (interval != value)
                {
                    interval = value;
                    NotifyPropertyChanged("Interval");
                }
            }
        }

        /// <summary>
        /// 13 个bytes
        /// </summary>
        /// <returns></returns>
        public byte[] ToBytes()
        {
            List<byte> buf = new List<byte>();
            buf.AddRange(BitConverter.GetBytes(No));
            buf.AddRange(BitConverter.GetBytes(Time.Ticks));
            buf.AddRange(BitConverter.GetBytes(On));
            buf.AddRange(BitConverter.GetBytes(Interval.Ticks));
            return buf.ToArray();
        }
        /// <summary>
        /// 13 个bytes
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public bool TryParse(byte[] value)
        {
            int cnt;
            return TryParse(value, 0, out cnt);
        }
        /// <summary>
        /// 13 个bytes
        /// </summary>
        /// <param name="value"></param>
        /// <param name="index"></param>
        /// <param name="cnt"></param>
        /// <returns></returns>
        public bool TryParse(byte[] value, int index, out int cnt)
        {
            cnt = 4 + 8 + 1 + 8;
            if (value.Length < cnt + index)
                return false;
            int idx = index;

            No = BitConverter.ToInt32(value, idx);
            idx += 4;

            try
            {
                Time = new DateTime(BitConverter.ToInt64(value, idx));
            }
            catch
            {
                return false;
            }
            idx += 8;

            On = BitConverter.ToBoolean(value, idx);
            idx++;

            try
            {
                Interval = TimeSpan.FromTicks(BitConverter.ToInt64(value, idx));
            }
            catch
            {
                return false;
            }
            idx += 8;
            return true;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="propertyname"></param>
        protected void NotifyPropertyChanged(string propertyname)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyname));
        }
        /// <summary>
        /// 
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
    }
    /// <summary>
    /// 信号列表返回
    /// </summary>
    public class GetSignListReponse : IPack
    {
        public int firstbm;
        public int lastbm;
        public List<SignData> datas = null;
        public byte[] ToBytes()
        {
            List<byte> buf = new List<byte>();
            buf.AddRange(BitConverter.GetBytes(firstbm));
            buf.AddRange(BitConverter.GetBytes(lastbm));

            if (datas != null)
            {
                buf.AddRange(BitConverter.GetBytes(datas.Count()));
                for (int i = 0; i < datas.Count(); i++)
                {
                    buf.AddRange(datas[i].ToBytes());
                }
            }
            else
            {
                buf.AddRange(BitConverter.GetBytes(0));
            }
            return buf.ToArray();
        }

        public bool TryParse(byte[] value)
        {
            int cnt = 4 + 4;
            if (value.Length < cnt)
                return false;
            int idx = 0;
            firstbm = BitConverter.ToInt32(value, idx);
            idx += 4;
            lastbm = BitConverter.ToInt32(value, idx);
            idx += 4;

            cnt += 4;
            if (value.Length < cnt)
                return false;
            int len = BitConverter.ToInt32(value, idx);
            idx += 4;
            if (len <= 0)
                datas = null;
            else
            {
                datas = new List<SignData>();
                for (int i = 0; i < len; i++)
                {
                    SignData f = new SignData();
                    int c;
                    if (!f.TryParse(value, idx, out c))
                    {
                        return false;
                    }
                    datas.Add(f);
                    idx += c;
                    cnt += c;
                }
            }
            return true;
        }
    }
}
